@@ -2,16 +2,20 @@
 import json
 import os
 import pathlib
+import re
 import shutil
 import sys
-import treelib
 from io import StringIO
+from typing import no_type_check
 
+import treelib  # type: ignore
 import yaml
 
 import liitos.gather as gat
 from liitos import ENCODING, log
 
+ALT_INJECTOR_HACK = 'INJECTED-ALT-TEXT-TO-TRIGGER-FIGURE-ENVIRONMENT-AROUND-IMAGE-IN-PANDOC'
+CAP_INJECTOR_HACK = 'INJECTED-CAP-TEXT-TO-MARK-MISSING-CAPTION-IN-OUTPUT'
 DOC_BASE = pathlib.Path('..', '..')
 STRUCTURE_PATH = DOC_BASE / 'structure.yml'
 SLASH = '/'
@@ -40,25 +44,33 @@ INCLUDE_SLOT = '\\include{'
 ![Alt Text Sting Red](other/images/red.png "Caption Text Sting Red")
 """
 IMG_LINE_STARTSWITH = '!['
+MD_IMG_PATTERN = re.compile(r'^!\[(?P<cap>[^(]*)\]\((?P<src>[^ ]+)\ *\"?(?P<alt>[^\"]*)\"?\)(?P<rest>.*)?$')
+MD_IMG_PATTERN_RIGHT_SPLIT = re.compile(r'^(?P<src>[^ ]+)\ *\"?(?P<alt>[^\"]*)\"?\)(?P<rest>.*)?$')
 
 
+@no_type_check
 class RedirectedStdout:
+    @no_type_check
     def __init__(self):
         self._stdout = None
         self._string_io = None
 
+    @no_type_check
     def __enter__(self):
         self._stdout = sys.stdout
         sys.stdout = self._string_io = StringIO()
         return self
 
+    @no_type_check
     def __exit__(self, type, value, traceback):
         sys.stdout = self._stdout
 
+    @no_type_check
     def __str__(self):
         return self._string_io.getvalue()
 
 
+@no_type_check
 def process_approvals(aspects: str) -> gat.Approvals | int:
     """TODO."""
     approvals_path = DOC_BASE / aspects[gat.KEY_APPROVALS]
@@ -83,6 +95,7 @@ def process_approvals(aspects: str) -> gat.Approvals | int:
     return approvals
 
 
+@no_type_check
 def process_binder(aspects: str) -> gat.Binder | int:
     """TODO."""
     bind_path = DOC_BASE / aspects[gat.KEY_BIND]
@@ -101,6 +114,8 @@ def process_binder(aspects: str) -> gat.Binder | int:
         handle.write('\n'.join(binder) + '\n')
     return binder
 
+
+@no_type_check
 def process_changes(aspects: str) -> gat.Changes | int:
     """TODO."""
     changes_path = DOC_BASE / aspects[gat.KEY_CHANGES]
@@ -124,6 +139,8 @@ def process_changes(aspects: str) -> gat.Changes | int:
             json.dump(changes, handle, indent=2)
     return changes
 
+
+@no_type_check
 def process_meta(aspects: str) -> gat.Meta | int:
     """TODO."""
     meta_path = DOC_BASE / aspects[gat.KEY_META]
@@ -142,7 +159,9 @@ def process_meta(aspects: str) -> gat.Meta | int:
         base_meta_path = DOC_BASE / metadata['document']['import']
         if not base_meta_path.is_file() or not base_meta_path.stat().st_size:
             log.error(
-                f'metadata declares import of base data from ({base_meta_path.name}) but failed to find non-empty base file at {base_meta_path}')
+                f'metadata declares import of base data from ({base_meta_path.name})'
+                f' but failed to find non-empty base file at {base_meta_path}'
+            )
             return 1
         with open(base_meta_path, 'rt', encoding=ENCODING) as handle:
             base_data = yaml.safe_load(handle)
@@ -154,29 +173,137 @@ def process_meta(aspects: str) -> gat.Meta | int:
     return metadata
 
 
+@no_type_check
+def parse_markdown_image(text_line: str) -> tuple[str, str, str, str]:
+    """Parse a markdown image line within our conventions into caption, src, alt, and optional rest."""
+    invalid_marker = ('', '', '', text_line)
+
+    exclam = '!'
+    osb = '['
+    if not text_line or not text_line.startswith(f'{exclam}{osb}'):
+        log.error(f'- INVALID-MD-IMG_LINE::START <<{text_line.rstrip()}>>')
+        return invalid_marker
+
+    csb = ']'
+    osb_cnt = text_line.count(osb)
+    csb_cnt = text_line.count(csb)
+    if osb_cnt + csb_cnt < 2:
+        log.error(f'- INVALID-MD-IMG_LINE::SB-TOK-CNT-LOW <<{text_line.rstrip()}>>')
+        return invalid_marker
+    if osb_cnt != csb_cnt:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::SB-TOK-CNT-UNBALANCED <<{text_line.rstrip()}>>')
+
+    orb = '('
+    cap_src_boundary = f'{csb}{orb}'
+    if cap_src_boundary not in text_line:
+        log.error(f'- INVALID-MD-IMG_LINE::CAP-SRC-BOUNDARY <<{text_line.rstrip()}>>')
+        return invalid_marker
+
+    crb = ')'
+    orb_cnt = text_line.count(orb)
+    crb_cnt = text_line.count(crb)
+    if orb_cnt + crb_cnt < 2:
+        log.error(f'- INVALID-MD-IMG_LINE::RB-TOK-CNT-LOW <<{text_line.rstrip()}>>')
+        return invalid_marker
+    if orb_cnt != crb_cnt:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::RB-TOK-CNT-UNBALANCED <<{text_line.rstrip()}>>')
+
+    quo = '"'
+    quo_cnt = text_line.count(quo)
+    if quo_cnt < 2:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::QU-TOK-CNT-LOW <<{text_line.rstrip()}>>')
+    if quo_cnt % 2:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::QU-TOK-CNT-UNBALANCED <<{text_line.rstrip()}>>')
+
+    sp = ' '
+    sp_cnt = text_line.count(sp)
+    if not sp_cnt:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::SP-TOK-CNT-LOW <<{text_line.rstrip()}>>')
+
+    dot = '.'
+    sla = '/'
+    abs_path_indicator = f'{csb}{orb}{sla}'
+    may_have_abs_path = abs_path_indicator in text_line
+    if may_have_abs_path:
+        log.info(f'- SUSPICIOUS-MD-IMG_LINE::MAY-HAVE-ABS-PATH <<{text_line.rstrip()}>>')
+    naive_upwards_path_indicator = f'{csb}{orb}{dot}{dot}{sla}'
+    may_have_upwards_path = naive_upwards_path_indicator in text_line
+    if may_have_upwards_path:
+        log.info(f'- SUSPICIOUS-MD-IMG_LINE::MAY-HAVE-UPWARDS-PATH <<{text_line.rstrip()}>>')
+
+    log.info('- parsing the markdown image text line ...')
+    if orb_cnt + crb_cnt > 2 or orb_cnt != crb_cnt:
+        # The regex is not safe for orb inside caption
+        left, right = text_line.split(cap_src_boundary, 1)
+        match_right = MD_IMG_PATTERN_RIGHT_SPLIT.match(right)
+        if not match_right:
+            log.error(f'- INVALID-MD-IMG_LINE::RE-MATCH-RIGHT-SPLIT-FAILED <<{text_line.rstrip()}>>')
+            return invalid_marker
+
+        parts = match_right.groupdict()
+        cap = left[2:]
+        if not cap:
+            log.warning(f'- INCOMPLETE-MD-IMG_LINE::CAP-MISS-INJECTED <<{text_line.rstrip()}>>')
+            cap = CAP_INJECTOR_HACK
+
+        src = parts['src']
+        alt = parts['alt']
+        rest = parts['rest']
+        if orb in alt or crb in alt:
+            log.warning(f'- MAYBE-MD-IMG_LINE::ALT-TRUNCATED-PARTIAL-MATCH <<{text_line.rstrip()}>>')
+            log.warning(f'  + parsed as ({cap=}, {src=}, {alt=}, {rest=}')
+
+        return cap, src, alt, rest
+
+    match = MD_IMG_PATTERN.match(text_line)
+    if not match:
+        log.error(f'- INVALID-MD-IMG_LINE::RE-MATCH-FAILED <<{text_line.rstrip()}>>')
+        return invalid_marker
+
+    parts = match.groupdict()
+    cap = parts['cap']
+    if not cap:
+        log.warning(f'- INCOMPLETE-MD-IMG_LINE::CAP-MISS-INJECTED <<{text_line.rstrip()}>>')
+        cap = CAP_INJECTOR_HACK
+
+    src = parts['src']
+    alt = parts['alt']
+    rest = parts['rest']
+    if orb in alt or crb in alt:
+        log.warning(f'- MAYBE-MD-IMG_LINE::ALT-TRUNCATED-FULL-MATCH <<{text_line.rstrip()}>>')
+        log.warning(f'  + parsed as ({cap=}, {src=}, {alt=}, {rest=}')
+
+    return cap, src, alt, rest
+
+
+@no_type_check
 def adapt_image(text_line: str, collector: list[str], upstream: str, root: str) -> str:
     """YES."""
-    before, xtr = text_line.split('](', 1)
-    has_caption = True if ' ' in xtr else False
-    img, after = xtr.split(' ', 1) if has_caption else xtr.split(')', 1)
-    img_path = str((pathlib.Path(upstream).parent / img).resolve()).replace(root, '')
+    cap, src, alt, rest = parse_markdown_image(text_line)
+    if not src:
+        log.error(f'parse of markdown image text line failed - empty src, and rest is <<{rest.rstrip()}>>')
+        return text_line
+
+    img_path = str((pathlib.Path(upstream).parent / src).resolve()).replace(root, '')
     collector.append(img_path)
     img_hack = img_path
     if f'/{IMAGES_FOLDER}' in img_path:
         img_hack = IMAGES_FOLDER + img_path.split(f'/{IMAGES_FOLDER}', 1)[1]
     elif f'/{DIAGRAMS_FOLDER}' in img_path:
         img_hack = DIAGRAMS_FOLDER + img_path.split(f'/{DIAGRAMS_FOLDER}', 1)[1]
+
     if img_hack != img_path:
-        log.debug(f'{img_hack} <--- OK? --- {img_path}')
-    return f'{before}]({img_hack}{" " if has_caption else ")"}{after}'
+        log.info(f'{img_hack} <--- OK? --- {img_path}')
+
+    alt_text = f'"{alt}"' if alt else f'"{ALT_INJECTOR_HACK}"'
+    belte_og_seler = f'![{cap}]({img_hack} {alt_text}){rest}'
+    log.info(f'==> belte-og-seler: ->>{belte_og_seler}<<-')
+    return belte_og_seler
 
 
+@no_type_check
 def harvest_include(
-    text_line: str,
-    slot: int,
-    regions: dict[str, list[tuple[tuple[int, int], str]]],
-    tree: treelib.Tree,
-    parent: str
+    text_line: str, slot: int, regions: dict[str, list[tuple[tuple[int, int], str]]], tree: treelib.Tree, parent: str
 ) -> None:
     """TODO."""
     include_local = text_line.split(INCLUDE_SLOT, 1)[1].rstrip('}').strip()
@@ -185,6 +312,7 @@ def harvest_include(
     tree.create_node(include, include, parent=parent)
 
 
+@no_type_check
 def rollup(
     jobs: list[list[str]],
     docs: dict[str, list[str]],
@@ -220,6 +348,7 @@ def rollup(
     return [[job for job in chain if job not in flat] for chain in jobs]
 
 
+@no_type_check
 def collect_assets(collector: list[str]) -> None:
     """TODO"""
     images = pathlib.Path(IMAGES_FOLDER)
@@ -238,6 +367,7 @@ def collect_assets(collector: list[str]) -> None:
             shutil.copy(source_asset, target_asset)
 
 
+@no_type_check
 def concatenate(
     doc_root: str | pathlib.Path, structure_name: str, target_key: str, facet_key: str, options: dict[str, bool]
 ) -> int:
@@ -256,7 +386,7 @@ def concatenate(
         doc_root=doc_root, structure_name=structure_name, target_key=target_key, facet_key=facet_key, command='concat'
     )
     log.info(f'prelude teleported processor into the document root at ({os.getcwd()}/)')
-    rel_concat_folder_path = pathlib.Path("render/pdf/")
+    rel_concat_folder_path = pathlib.Path('render/pdf/')
     rel_concat_folder_path.mkdir(parents=True, exist_ok=True)
     os.chdir(rel_concat_folder_path)
     log.info(f'concatenate (this processor) teleported into the render/pdf location ({os.getcwd()}/)')
@@ -388,7 +518,9 @@ def concatenate(
                             continue
                     if in_region:
                         if line.startswith(READ_SLOT_CONTEXT_BEGIN):
-                            sub_include = line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
+                            sub_include = (
+                                line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
+                            )
                             sub_include = str(pathlib.Path(include).parent / sub_include)
                         elif line.startswith(READ_SLOT_FENCE_END):
                             end = slot
@@ -420,7 +552,9 @@ def concatenate(
                                 continue
                         if in_region:
                             if line.startswith(READ_SLOT_CONTEXT_BEGIN):
-                                sub_sub_include = line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
+                                sub_sub_include = (
+                                    line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
+                                )
                                 sub_sub_include = str(pathlib.Path(sub_include).parent / sub_sub_include)
                             elif line.startswith(READ_SLOT_FENCE_END):
                                 end = slot
@@ -440,7 +574,9 @@ def concatenate(
                         sub_sub_sub_include = ''
                         for slot, line in enumerate(documents[sub_sub_include]):
                             if line.startswith(IMG_LINE_STARTSWITH):
-                                documents[sub_sub_include][slot] = adapt_image(line, img_collector, sub_sub_include, root_path)
+                                documents[sub_sub_include][slot] = adapt_image(
+                                    line, img_collector, sub_sub_include, root_path
+                                )
                             log.debug(f'{slot :02d}|{line.rstrip()}')
                             if not in_region:
                                 if line.startswith(READ_SLOT_FENCE_BEGIN):
@@ -452,8 +588,12 @@ def concatenate(
                                     continue
                             if in_region:
                                 if line.startswith(READ_SLOT_CONTEXT_BEGIN):
-                                    sub_sub_sub_include = line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
-                                    sub_sub_sub_include = str(pathlib.Path(sub_sub_include).parent / sub_sub_sub_include)
+                                    sub_sub_sub_include = (
+                                        line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
+                                    )
+                                    sub_sub_sub_include = str(
+                                        pathlib.Path(sub_sub_include).parent / sub_sub_sub_include
+                                    )
                                 elif line.startswith(READ_SLOT_FENCE_END):
                                     end = slot
                                     insert_regions[sub_sub_include].append(((begin, end), sub_sub_sub_include))
@@ -486,12 +626,21 @@ def concatenate(
                                         continue
                                 if in_region:
                                     if line.startswith(READ_SLOT_CONTEXT_BEGIN):
-                                        sub_sub_sub_sub_include = line.replace(READ_SLOT_CONTEXT_BEGIN, '').split(')', 1)[0].strip("'").strip('"')
-                                        sub_sub_sub_sub_include = str(pathlib.Path(sub_sub_sub_include).parent / sub_sub_sub_sub_include)
+                                        sub_sub_sub_sub_include = (
+                                            line.replace(READ_SLOT_CONTEXT_BEGIN, '')
+                                            .split(')', 1)[0]
+                                            .strip("'")
+                                            .strip('"')
+                                        )
+                                        sub_sub_sub_sub_include = str(
+                                            pathlib.Path(sub_sub_sub_include).parent / sub_sub_sub_sub_include
+                                        )
                                     elif line.startswith(READ_SLOT_FENCE_END):
                                         end = slot
                                         insert_regions[sub_sub_sub_include].append(((begin, end), sub_sub_sub_include))
-                                        tree.create_node(sub_sub_sub_sub_include, sub_sub_sub_sub_include, parent=sub_sub_sub_include)
+                                        tree.create_node(
+                                            sub_sub_sub_sub_include, sub_sub_sub_sub_include, parent=sub_sub_sub_include
+                                        )
                                         in_region = False
                                         begin, end = 0, 0
                                         sub_sub_sub_sub_include = ''
@@ -516,7 +665,7 @@ def concatenate(
         log.info(f'dependencies for the {len(insert_regions)} document parts:')
         for key, regions in insert_regions.items():
             num_in = len(regions)
-            dashes = "-" * num_in
+            dashes = '-' * num_in
             incl_disp = f'( {num_in} include{"" if num_in == 1 else "s"} )'
             indicator = '(no includes)' if not regions else f'<{dashes + incl_disp + dashes}'
             log.info(f'- part {key} {indicator}')
