@@ -1,7 +1,7 @@
 """Apply all pairs in patch module on document."""
 from collections.abc import Iterable
 
-from liitos import ENCODING, log
+from liitos import log
 
 TAB_START_TOK = r'\begin{longtable}[]{@{}'
 TOP_RULE = r'\toprule()'
@@ -110,12 +110,12 @@ def patch(incoming: Iterable[str]) -> list[str]:
     for table in table_ranges:
         from_here = table['start']
         thru_there = table['amend']
-        print('Table:')
-        print(f'-from {incoming[from_here]}')
-        print(f'-thru {incoming[thru_there]}')
+        log.info('Table:')
+        log.info(f'-from {incoming[from_here]}')  # type: ignore
+        log.info(f'-thru {incoming[thru_there]}')  # type: ignore
         on_off = (from_here, thru_there + 1)
         on_off_slots.append(on_off)
-        tables_in.append((on_off, [line for line in incoming[on_off[0] : on_off[1]]]))
+        tables_in.append((on_off, [line for line in incoming[on_off[0] : on_off[1]]]))  # type: ignore
 
     log.debug('# - - - 8< - - -')
     if tables_in:
