@@ -2,7 +2,7 @@
 import pathlib
 import pkgutil
 
-from liitos import ENCODING, log
+from liitos import ENCODING
 
 RESOURCES = (
     'templates/approvals.yml',
@@ -12,6 +12,7 @@ RESOURCES = (
     'templates/meta.yml',
     'templates/meta-patch.yml',
     'templates/metadata.tex.in',
+    'templates/mkdocs.yml.in',
     'templates/publisher.tex.in',
     'templates/setup.tex.in',
     'templates/vocabulary.yml',
@@ -24,7 +25,7 @@ def load_resource(resource: str, is_complete_path: bool = False) -> str:
         with open(resource, 'rt', encoding=ENCODING) as handle:
             return handle.read()
     else:
-        return pkgutil.get_data(__package__, resource).decode(encoding=ENCODING)
+        return pkgutil.get_data(__package__, resource).decode(encoding=ENCODING)  # type: ignore
 
 
 def eject(argv: list[str] | None = None) -> int:
@@ -37,9 +38,8 @@ def eject(argv: list[str] | None = None) -> int:
     (into_path / 'templates').mkdir(parents=True, exist_ok=True)
     for resource in RESOURCES:
         write_to = into_path / resource
-        data = pkgutil.get_data(__package__, resource).decode(encoding=ENCODING)
+        data = pkgutil.get_data(__package__, resource).decode(encoding=ENCODING)  # type: ignore
         with open(write_to, 'wt', encoding=ENCODING) as target:
             target.write(data)
 
     return 0
-
